#pragma rtGlobals = 1		// modern global method

#include <Strings as Lists>
#include <Keyword-Value>

// PPT Utilites - Common utility macros
// part of the PPT, the "Patcher's Power Tools"
//  09.08.96 Francisco Mendez

// 29.10.96 added 3 procedures: DrawScaleBars, DrawZeroLine and DrawEvent
// 04.12.96 added 2 string functions: PPTGetXAxes and PPTGetYAxes
// 04.12.96 there was a bug in DrawScaleBars, if only one bar had to be drawn
// 04.12.96 rename/kill waves compares wavenames case-insensitive
// 10.07.97 new macro: Duplicate/clone top window
// 27.10.97 changed most procedures into functions, echoing procedure calls

// ****************************************************

Menu "Macros"
	Submenu "PPT Macros"
		"Rename multiple Waves...",		PPTRenameMultipleWaves ()
		"Kill multiple Waves...",		PPTKillMultipleWaves ()
		"-"	
		"Set Cursor A to NaN/1",		PPTKillCursorPointA ()
		"Set Cursor B to NaN/2",		PPTKillCursorPointB ()
		"Set Cursor Range to NaN",		PPTKillCursorRange ()
		"-"
		"Draw Scale Bars...",			PPTDrawScaleBars ()
		"Draw Zero Line...",				PPTDrawZeroLine ()
		"Draw Event...",					PPTDrawEvent ()
		"-"
		"Duplicate top window",			PPTDuplicateWindow ()
		"Show Axes and Labels",			PPTShowAxes ()
		"Hide Axes and Labels",			PPTHideAxes ()
		"Transparent Layout Objects",	PPTTransparentObjects ()
		"Set and scroll Cursor...",		PPTSetCursor ()
		"-"
		"Make logarithmic X-Wave...",	PPTMakeLogXWave ()
	end
end

// ****************************************************
// Draw x- and y-scale bars
// ****************************************************

Proc PPTDrawScaleBars (xAxis, yAxis, xOrient, yOrient, x0, y0, xSize, ySize, xLabel, yLabel)
	String xAxis, yAxis, xLabel, yLabel
	Variable yOrient, xOrient, y0, x0, ySize, xSize
	Prompt xAxis, "Choose bottom Axis", popup, PPTGetXAxes ()
	Prompt yAxis, "Choose left Axis", popup, PPTGetYAxes ()
	Prompt xOrient, "Orientation of x-Scalebar", popup, "bottom;top;none"
	Prompt yOrient, "Orientation of y-Scalebar", popup, "left;right;none"
	Prompt x0, "Left point of  x-Scalebar"
	Prompt y0, "Bottom point of  y-Scalebar"
	Prompt xSize, "Size of  x-Scalebar"
	Prompt ySize, "Size of y-Scalebar"
	Prompt xLabel, "Label of x-Axis"
	Prompt yLabel, "Label of y-Axis"

	PauseUpdate; Silent 1

	if (((ySize == 0) %& (xSize == 0)) %| ((xOrient == 3) %& (yOrient == 3)))
		Abort "Please select at least one axis and set its size to a value > 0!"
	endif

	String echo				// Igor does not generate History with parameters
	sprintf echo, "PPTDrawScaleBars (\"%s\", \"%s\", %d, %d, %g, %g, %g, %g, \"%s\", \"%s\")", xAxis, yAxis, xOrient, yOrient, x0, y0, xSize, ySize, xLabel, yLabel
	print echo

	if (xOrient == 2)		// x-bar at top
		y0 += ySize
	endif
	if (yOrient == 2)		// y-bar at right
		x0 += xSize
	endif

	SetDrawLayer UserFront

	// Draw x-Scale Bar
	SetDrawEnv xcoord=$xAxis, ycoord=$yAxis
	if ((xOrient != 3) %& (xSize > 0))
		if (yOrient == 2)
			DrawLine x0, y0, x0 - xSize, y0
		else
			DrawLine x0, y0, x0 + xSize, y0
		endif
	endif

	// Draw y-Scale Bar
	SetDrawEnv xcoord = $xAxis, ycoord = $yAxis
	if ((yOrient != 3) %& (ySize > 0))
		if (xOrient == 2)
			DrawLine x0, y0, x0, y0 - ySize
		else
			DrawLine x0, y0, x0, y0 + ySize
		endif
	endif

	Variable xJust, yJust
	Variable xPos, yPos
	String label

	// Draw x-Axis Text
	if ((xOrient != 3) %& (xSize > 0))
		xJust = 1			// centered text
		yPos = y0
		if (xOrient == 1)
			yJust = 2		// top justification
		endif
		if (xOrient == 2)
			yJust = 0		// bottom justification
		endif
		if (yOrient == 2)
			xPos = x0 - xSize / 2
		else
			xPos = x0 + xSize / 2
		endif
		label = num2str (xSize)
		if (strlen (xLabel) > 0)
			label += " "
			label += xLabel
		endif
		SetDrawEnv xcoord = $xAxis, ycoord = $yAxis, textxjust = xJust, textyjust = yJust
		DrawText xPos, yPos, label
	endif

	// Draw y-Axis Text
	if ((yOrient != 3) %& (ySize > 0))
		yJust = 1			// mid justified text
		xPos = x0
		if (yOrient == 1)
			xJust = 2		// right justification
		endif
		if (yOrient == 2)
			xJust = 0		// left justification
		endif
		if (xOrient == 2)
			yPos = y0 - ySize / 2
		else
			yPos =y0 + ySize / 2
		endif
		label = num2str (ySize)
		if (strlen (yLabel) > 0)
			label += " "
			label += yLabel
		endif
		SetDrawEnv xcoord = $xAxis, ycoord = $yAxis, textxjust = xJust, textyjust = yJust, textrot = 90
		DrawText xPos, yPos, label
	endif

	ResumeUpdate
End Proc		// PPTDrawScaleBars

// ****************************************************
// returns a list of all x-axes (top/bottom) in the graph axes, sorted by axisType
// ****************************************************

Function /S PPTGetXAxes ()
	String		theAxisList, theAxisName
	String		theBottomList, theTopList
	String		theAxisType
	String		theList
	Variable	i, len

	theAxisList = AxisList ("")
	i = 0
	theBottomList = ""
	theTopList = ""

	do
		theAxisName = GetStrFromList (theAxisList, i, ";")
		if (strlen (theAxisName) == 0)					//end of list found
			break
		endif
		theAxisType = StrByKey ("AXTYPE", AxisInfo ("", theAxisName))

		if (cmpstr (theAxisType, "top") == 0)
			theTopList += (theAxisName + ";")
		endif
		if (cmpstr (theAxisType, "bottom") == 0)
			theBottomList += (theAxisName + ";")
		endif

		i += 1
	while (1)

	theList = ""
	if (strlen (theTopList) > 0)
		theList += (theTopList + "-;")
	endif
	if (strlen (theBottomList) > 0)
		theList += (theBottomList + "-;")
	endif

	len = strlen (theList)
	if (len > 0)
		theList [len -2, len - 1] = ""
	endif

	return (theList)
End

// ****************************************************
// returns a list of all y-axes (left/right) in the graph axes, sorted by axisType
// ****************************************************

Function /S PPTGetYAxes ()
	String		theAxisList, theAxisName
	String		theLeftList, theRightList
	String		theAxisType
	String		theList
	Variable	i, len

	theAxisList = AxisList ("")
	i = 0
	theLeftList = ""
	theRightList = ""

	do
		theAxisName = GetStrFromList (theAxisList, i, ";")
		if (strlen (theAxisName) == 0)					//end of list found
			break
		endif
		theAxisType = StrByKey ("AXTYPE", AxisInfo ("", theAxisName))

		if (cmpstr (theAxisType, "left") == 0)
			theLeftList += (theAxisName + ";")
		endif
		if (cmpstr (theAxisType, "right") == 0)
			theRightList += (theAxisName + ";")
		endif

		i += 1
	while (1)

	theList = ""
	if (strlen (theLeftList) > 0)
		theList += (theLeftList + "-;")
	endif
	if (strlen (theRightList) > 0)
		theList += (theRightList + "-;")
	endif

	len = strlen (theList)
	if (len > 0)
		theList [len -2, len - 1] = ""
	endif

	return (theList)
End

// ****************************************************
// Draw a red zero line
// ****************************************************

Proc PPTDrawZeroLine (yAxis, pos)
	String yAxis
	Variable pos = 0
	Prompt pos, "y-Position of Zeroline"
	Prompt yAxis, "Choose Left Axis", popup, PPTGetYAxes ()

	PauseUpdate; Silent 1

	SetDrawEnv ycoord = $yAxis, linefgc = (65535,16385,16385), dash = 1
	DrawLine 0, pos, 1, pos
	String echo
	sprintf echo "PPTDrawZeroLine (\"%s\", %g)", yAxis, pos
	print echo

	ResumeUpdate
End Proc		// PPTDrawZeroLine

// ****************************************************
// Draw an event into the top graph
// ****************************************************

Proc PPTDrawEvent (evntStart, evntEnd, axis, mode, text)
	Variable	evntStart, evntEnd
	String		axis
	Variable	mode
	String		text
	Prompt 	evntStart, "Begin of Event:"
	Prompt		evntEnd, "End of Event:"
	Prompt 	axis, "Choose Bottom Axis", popup, PPTGetXAxes ()
	Prompt 	mode, "Select mode", popup, "Rect;Line"
	Prompt		text, "Label"

	Silent 1; PauseUpdate

	String echo
	sprintf echo, "PPTDrawEvent (%g, %g, \"%s\", %d, \"%s\")", evntStart, evntEnd, axis, mode, text
	print echo

	SetDrawLayer UserBack
	if (strlen (text) > 0)
		SetDrawEnv xcoord = $axis, textxjust = 1
		DrawText ((evntStart + evntEnd) / 2), 0, text
		ModifyGraph margin (top) = 24
	endif
	if (mode == 2)
		SetDrawEnv xcoord = $axis, linethick= 3.00
		DrawLine evntStart, 0, evntEnd, 0
	else
		SetDrawEnv xcoord = $axis
		SetDrawEnv fillfgc = (49151,49152,65535), fillpat = 3
		SetDrawEnv linefgc = (49151,49152,65535), linepat = 3
		DrawRect evntStart, 0, evntEnd, 1
	endif

	ResumeUpdate
End Proc		// PPTDrawEvent

// ****************************************************
// Rename multiple waves matching "subStr"
// ****************************************************

Proc PPTRenameMultipleWaves (subStr, replaceStr, prefix, suffix)
	String subStr = "wave"
	String replaceStr = ""
	String prefix = ""
	String suffix = ""
	Prompt subStr, "Replace this substring from each wavename"
	Prompt replaceStr, "by this string"
	Prompt prefix, "Add this Prefix"
	Prompt suffix, "Append this suffix"

	Silent 1; PauseUpdate

	PPTDoRenameMultipleWaves (subStr, replaceStr, prefix, suffix)

	String echo	// Igor does not generate right command in the history
	sprintf echo "PPTDoRenameMultipleWaves (\"%s\", \"%s\", \"%s\", \"%s\")", subStr, replaceStr, prefix, suffix
	print echo
End Proc		// PPTRenameMultipleWaves

// ****************************************************

Function PPTDoRenameMultipleWaves (subStr, replaceStr, prefix, suffix)
	String subStr
	String replaceStr
	String prefix
	String suffix

	String matchStr = "*" + subStr + "*"
	String theWaveList = WaveList (matchStr, ";","")

	String theOldWave, theNewWave, beforeSubStr, afterSubStr
	Variable index = 0
	Variable renamed = 0
	Variable pos
	
	do
		// get the next wave name
		theOldWave = GetStrFromList (theWaveList, index, ";")
		if (strlen (theOldWave) == 0)
			break
		endif
		pos = strsearch (LowerStr (theOldWave), LowerStr (subStr), 0)
		if (pos >= 0)
			beforeSubStr = theOldWave [0, pos - 1]
			afterSubStr = theOldWave [pos + strlen (subStr), strlen (theOldWave)]
			theNewWave = prefix + beforeSubStr + replaceStr+ afterSubStr + suffix
			if (strlen (theNewWave) > 0)
				Rename $theOldWave, $theNewWave
				renamed += 1
			endif
		endif
		index += 1
	while (1)

	print "Waves renamed: ", renamed
	ResumeUpdate 
End	Function	// PPTDoRenameMultipleWaves

// ****************************************************
// Kill or keep all waves matching "subStr"
// ****************************************************

Proc PPTKillMultipleWaves (subStr, mode)
	String subStr = "wave"
	Variable mode
	Prompt subStr, "Search for waves containing this substring"
	Prompt mode, "Mode", popup, "Kill matching waves;Keep matching waves"

	Silent 1; PauseUpdate

	PPTDoKillMultipleWaves (subStr, mode)
	String echo	// Igor does not generate right command in the history
	sprintf echo "PPTDoKillMultipleWaves (\"%s\", %d)", subStr, mode
	print echo
End Proc		// PPTKillMultipleWaves

// ****************************************************

Function PPTDoKillMultipleWaves (subStr, mode)
	String subStr
	Variable mode

	String theWaveList = WaveList ("*", ";","")
	String theWaveName
	Variable index = 0
	Variable killed = 0
	Variable pos

	do
		// get the next wave name
		theWaveName = GetStrFromList (theWaveList, index, ";")
		if (strlen (theWaveName) == 0)
			break
		endif
		pos = strsearch (LowerStr (theWaveName), LowerStr (subStr), 0)
		if  (((pos >= 0) %& (mode == 1)) %| ((pos == -1) %& (mode == 2)))
			KillWaves $theWaveName
			killed += 1
		endif
		index += 1
	while (1)

	print "Waves killed: ", killed
End		// PPTDoKillMultipleWaves

// ****************************************************
// Set Point of Cursor A to NaN
// ****************************************************

Function PPTKillCursorPointA ()
	PPTKillCursorPoint ("A")
	DoUpdate
End Function		// PPTKillCursorPointA

// ****************************************************
// Set Point of Cursor B to NaN
// ****************************************************

Function PPTKillCursorPointB ()
	PPTKillCursorPoint ("B")
	DoUpdate
End Function		// PPTKillCursorPointB

// ****************************************************

Function PPTKillCursorPoint (cursor)
	String cursor

	String theString = CsrWave ($cursor)
	Wave/D theWave = $theString
	Variable thePoint = pcsr ($cursor)

	theWave [thePoint] = NaN
End Function		// PPTKillCursorPoint

// ****************************************************
// Set Cursor Range to NaN
// ****************************************************

Function PPTKillCursorRange ()
	String theWaveName
	Variable min, max, temp

	theWaveName = CsrWave (A)
	if (CmpStr (theWaveName, CsrWave (B)))
		Abort "Please locate both Cursors on the same wave!"
	endif

	WAVE theWave = $theWaveName
	min = pcsr (A)
	max = pcsr (B)

	if (max < min)
		temp = min
		min = max
		max = temp
	endif

	do
		theWave [min] = NaN
		min += 1
	while (min <= max)

	DoUpdate
End Function		// PPTKillCursorRange

// ****************************************************
// Duplicate topmost graph, table or layout window
// ****************************************************

Function PPTDuplicateWindow ()
	String topWinName, theCmd

	Silent 1; PauseUpdate

	topWinName = WinName (0, 1 + 2 + 4)		// Get top graph, table or layout
	if (strlen (topWinName) <= 0)
		Abort "There's no graph, table or layout to duplicate!"
	endif
	theCmd = "DoWindow/R " + topWinName		// Save recreation macro
	execute theCmd
	theCmd = topWinName + "()"				// open cloned window
	execute theCmd

	ResumeUpdate
End Proc		// PPTDuplicateWindow

// ****************************************************
//Show/Hide Axes
// ****************************************************

Function PPTShowAxes ()
	if (strlen (WinName (0, 1)) <= 0)
		Abort "There's no graph!"
	endif

	ModifyGraph axThick = 1
	ModifyGraph zeroThick = 1
	ModifyGraph noLabel = 0

	DoUpdate
End Proc		// PPTShowAxes

// ****************************************************

Function PPTHideAxes ()
	if (strlen (WinName (0, 1)) <= 0)
		Abort "There's no graph!"
	endif

	ModifyGraph axThick = 0
	ModifyGraph zeroThick = 1
	ModifyGraph noLabel = 2

	DoUpdate
End Proc		// PPTHideAxes

// ****************************************************
//All Layout-Elements transparent without frame
// ****************************************************

Proc PPTTransparentObjects ()
	Silent 1; PauseUpdate
	ModifyLayout frame=0
	ModifyLayout trans=1
	ResumeUpdate
End	Proc	// PPTTransparentObjects

// ****************************************************
//Make an X-Wave with evenly spaced logarithmic values
// ****************************************************

Proc PPTMakeLogXWave (waveName, numPts, startVal, endVal, overWrite)
	String waveName = "fit"
	Variable numPts = 400
	Variable /D startVal = 1e-9
	Variable /D endVal = 1e-5
	Variable overWrite = 2
	Prompt waveName, "Wavename"
	Prompt numPts, "Number of points"
	Prompt startVal, "Start value"
	Prompt endVal, "End value"
	Prompt overWrite, "Overwrite existing waves", popup, "Yes;No"

	Silent 1; PauseUpdate
	PPTDoMakeLogXWave (waveName, numPts, startVal, endVal, overWrite)
End

// ****************************************************

Function PPTDoMakeLogXWave (waveName, numPts, startVal, endVal, overWrite)
	String		waveName
	Variable	numPts
	Variable /D	startVal
	Variable /D	endVal
	Variable	overWrite

	String theString = "Make"
	if (overWrite == 1)
		theString += "/O"
	endif
	theString +="/N=" + num2str (numPts) + "/D " + waveName + "_x"
	Execute theString

	theString = "Make"
	if (overWrite == 1)
		theString += "/O"
	endif
	theString += "/N=" + num2str (numPts) + "/D " + waveName + "_y"
	Execute theString

	sprintf theString, "SetScale/I x %f,%f,\"\", %s_x", log (startVal), log (endVal), waveName
	Execute theString

	theString = waveName + "_x" + " = x"
	Execute theString

	sprintf theString, "SetScale x 0,%d,\"\", %s_x", numPts, waveName
	Execute theString

	sprintf theString, "%s_x = 10^%s_x", waveName, waveName
	Execute theString
End Function		// PPTDoMakeLogXWave
